/************************/
/*    Defines		    */
/************************/
typedef unsigned char u8_t;
typedef unsigned short int u16_t;
typedef unsigned long int u32_t;

typedef signed char s8_t;
typedef signed short int s16_t;
typedef signed long int s32_t;

typedef unsigned long int mem_ptr_t;

#define U16_F "u"
#define S16_F "d"
#define X16_F "x"
#define U32_F "u"
#define S32_F "d"
#define X32_F "x"
#define SZT_F "d"

#define BYTE_ORDER BIG_ENDIAN
#define PACK_STRUCT_STRUCT

#define FIXED_TCP_BUILD_MSS_OPTION(x) (x) = htonl(((u32_t)2) | \
												  ((u32_t)4 << 8) | \
												  (((u32_t)TCP_MSS / 256) << 16) | \
												  (TCP_MSS & 255) << 24)

/************************/
/*    Include files     */
/************************/
#include "lwip/tcp.h"

/************************/
/*  Declarations    	*/
/************************/
struct hdropts {
struct tcp_hdr tcphdr;  	 /* the pointer to the TCP header */
u32_t opts;
};

struct tcp_hdr *tcphdr;  	 /* the pointer to the TCP header */
static void tcp_parseopt(struct tcp_pcb *pcb);

/************************/
/*    Global Variables	*/
/************************/

int main(void)
{
	struct hdropts test_hdropts;  /* the TCP packet */
	struct tcp_pcb test_tcppcb;  /* the TCP PCB */

	memset(&test_hdropts, 0x0, sizeof(struct hdropts));
	memset(&test_tcppcb, 0x0, sizeof(struct tcp_pcb));

	/* options flag */
	u8_t optflags;
	optflags = TF_SEG_OPTS_MSS;

	/* options length */
	u8_t optlen;
    optlen = LWIP_TCP_OPT_LENGTH(optflags);

    struct tcp_hdr *test_tcphdr;  /* the TCP header */
    test_tcphdr = (struct tcp_hdr *)&test_hdropts;

    /* hdr length + options length = 6 */
    TCPH_HDRLEN_SET(test_tcphdr, (5 + optlen / 4));

    /* write MSS option */
	u32_t *opts;
    opts = (u32_t *)(test_tcphdr + 1);
    TCP_BUILD_MSS_OPTION(*opts);

    /* uncomment to verify the test */
    // FIXED_TCP_BUILD_MSS_OPTION(*opts);

    tcphdr = test_tcphdr;

    /* try to parse the option */
    tcp_parseopt(&test_tcppcb);

	return 0;
}

/************************/
/*  Stubbed Functions   */
/************************/
static void
tcp_parseopt(struct tcp_pcb *pcb)
{
  u16_t c, max_c;
  u16_t mss;
  u8_t *opts, opt;
#if LWIP_TCP_TIMESTAMPS
  u32_t tsval;
#endif

  opts = (u8_t *)tcphdr + TCP_HLEN;

  /* Parse the TCP MSS option, if present. */
  if(TCPH_HDRLEN(tcphdr) > 0x5) {
    max_c = (TCPH_HDRLEN(tcphdr) - 5) << 2;
    for (c = 0; c < max_c; ) {
      opt = opts[c];
      switch (opt) {
      case 0x00:
        /* End of options. */
        LWIP_DEBUGF(TCP_INPUT_DEBUG, ("tcp_parseopt: EOL\n"));
        return;
      case 0x01:
        /* NOP option. */
        ++c;
        LWIP_DEBUGF(TCP_INPUT_DEBUG, ("tcp_parseopt: NOP\n"));
        break;
      case 0x02:
        LWIP_DEBUGF(TCP_INPUT_DEBUG, ("tcp_parseopt: MSS\n"));
        if (opts[c + 1] != 0x04 || c + 0x04 > max_c) {
          /* Bad length */
          LWIP_DEBUGF(TCP_INPUT_DEBUG, ("tcp_parseopt: bad length\n"));
          return;
        }
        /* An MSS option with the right option length. */
        mss = (opts[c + 2] << 8) | opts[c + 3];
        /* Limit the mss to the configured TCP_MSS and prevent division by zero */
        pcb->mss = ((mss > TCP_MSS) || (mss == 0)) ? TCP_MSS : mss;
        /* Advance to next option */
        c += 0x04;
        break;
#if LWIP_TCP_TIMESTAMPS
      case 0x08:
        LWIP_DEBUGF(TCP_INPUT_DEBUG, ("tcp_parseopt: TS\n"));
        if (opts[c + 1] != 0x0A || c + 0x0A > max_c) {
          /* Bad length */
          LWIP_DEBUGF(TCP_INPUT_DEBUG, ("tcp_parseopt: bad length\n"));
          return;
        }
        /* TCP timestamp option with valid length */
        tsval = (opts[c+2]) | (opts[c+3] << 8) |
          (opts[c+4] << 16) | (opts[c+5] << 24);
        if (flags & TCP_SYN) {
          pcb->ts_recent = ntohl(tsval);
          pcb->flags |= TF_TIMESTAMP;
        } else if (TCP_SEQ_BETWEEN(pcb->ts_lastacksent, seqno, seqno+tcplen)) {
          pcb->ts_recent = ntohl(tsval);
        }
        /* Advance to next option */
        c += 0x0A;
        break;
#endif
      default:
        LWIP_DEBUGF(TCP_INPUT_DEBUG, ("tcp_parseopt: other\n"));
        if (opts[c + 1] == 0) {
          LWIP_DEBUGF(TCP_INPUT_DEBUG, ("tcp_parseopt: bad length\n"));
          /* If the length field is zero, the options are malformed
             and we don't process them further. */
          return;
        }
        /* All other options have a length field, so that we easily
           can skip past them. */
        c += opts[c + 1];
      }
    }
  }
}
